from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from .account_api import AccountApi
from .auction_api import AuctionApi
from .block_api import BlockApi
from .chain_api import ChainApi
from .connection_api import ConnectionApi
from .contract_api import ContractApi
from .leaderboard_api import LeaderboardApi
from .nexus_api import NexusApi
from .organization_api import OrganizationApi
from .platform_api import PlatformApi
from .rpc_api import RpcApi
from .sale_api import SaleApi
from .storage_api import StorageApi
from .token_api import TokenApi
from .transaction_api import TransactionApi
from .validator_api import ValidatorApi



# coding: utf-8

# flake8: noqa
"""
    Phantasma API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import models into model package
from .abi_event_result import ABIEventResult
from .abi_method_result import ABIMethodResult
from .abi_parameter_result import ABIParameterResult
from .account_result import AccountResult
from .address import Address
from .address_kind import AddressKind
from .archive_result import ArchiveResult
from .auction_result import AuctionResult
from .balance_result import BalanceResult
from .block_result import BlockResult
from .chain_result import ChainResult
from .contract_result import ContractResult
from .crowdsale_result import CrowdsaleResult
from .event_result import EventResult
from .governance_result import GovernanceResult
from .interop_result import InteropResult
from .leaderboard_result import LeaderboardResult
from .leaderboard_row_result import LeaderboardRowResult
from .net_info_peer import NetInfoPeer
from .net_info_peer_connection_status import NetInfoPeerConnectionStatus
from .net_info_peer_connection_status_channel import NetInfoPeerConnectionStatusChannel
from .net_info_peer_connection_status_monitor import NetInfoPeerConnectionStatusMonitor
from .nexus_result import NexusResult
from .node_info import NodeInfo
from .node_info_other import NodeInfoOther
from .node_info_protocol_version import NodeInfoProtocolVersion
from .oracle_result import OracleResult
from .organization_result import OrganizationResult
from .paginated_result import PaginatedResult
from .platform_result import PlatformResult
from .pub_key import PubKey
from .response_query import ResponseQuery
from .result_abci_query import ResultAbciQuery
from .result_health import ResultHealth
from .result_net_info import ResultNetInfo
from .result_status import ResultStatus
from .result_status_sync_info import ResultStatusSyncInfo
from .result_status_validator_info import ResultStatusValidatorInfo
from .rpc_request import RpcRequest
from .rpc_response import RpcResponse
from .script_result import ScriptResult
from .signature_result import SignatureResult
from .stake_result import StakeResult
from .storage_result import StorageResult
from .token_data_result import TokenDataResult
from .token_external_result import TokenExternalResult
from .token_price_result import TokenPriceResult
from .token_property_result import TokenPropertyResult
from .token_result import TokenResult
from .token_series_result import TokenSeriesResult
from .transaction_result import TransactionResult
from .validator_result import ValidatorResult
from .validator_settings import ValidatorSettings



from datetime import datetime
import binascii
from ecpy.curves import Curve, Point
from ecpy.keys import ECPublicKey, ECPrivateKey
from ecpy.eddsa import EDDSA
import hashlib

from ..Types.Extensions import PBinaryWriter

from ..VM.ScriptBuilder import ScriptBuilder


class Transaction():

    '''
    The Transaction object implements all the basic methods needed to build a Phantasma Transaction string
    and sign it with HEX PK in order to broadcast to the Phantasma Blockchain.

    Args:
        nexusName (str): mainnet (production) or testnet (testing).
        chainName (str): chain name (main).
        script (str): script to execute.
        expiration (int): expiration timestamp.
        payload (str): payload message.

    Attributes:
        nexusName (str): mainnet (production) or testnet (testing).
        chainName (str): chain name (main).
        script (str): script to execute.
        expiration (int): expiration timestamp.
        payload (str): payload message.
        signatures (list): list of signatures, at least one is needed to broadcast tx.
    '''

    nexusName: str = None
    chainName: str = None
    script: str = None
    expiration: int = None
    payload: str = None
    hash: str = None

    signatures = []

    def __init__(
            self,
            nexusName: str,
            chainName: str,
            script: str,
            expiration: int,
            payload: str = 'PHANPY-1.0') -> None:

        if expiration is None:
            now = datetime.now()
            expiration = int(datetime.timestamp(now)) + 1000

        self.nexusName = nexusName
        self.chainName = chainName
        self.script = script
        self.expiration = expiration
        self.payload = payload
        self.signatures = []

    def sign(self, pk: int) -> None:
        '''Sign the transaction with HEX PK of the Sender's Address.

        Args:
            pk (int): HEX private key. Example: 0x411d7dabb39b455aadc49897e2fa13234585116d4f5eee198105f33f8f62d0b8

        Returns:
            None (The generated signature is added to the signatures list)
        '''
        signature = self.getSign(self.toString(False), pk)
        self.signatures.insert(0, signature)

    def toString(self, withSignature: bool) -> str:
        '''By using a ScriptBuilder object this method generates the needed string to broadcast the tx to
        the Phanstasma Blockchain.

        Args:
            withSignature (bool): flags that indicates if need to attach signatures or not to the end result.

        Returns:
            toString (str): Transaction object as string.
        '''

        num = self.expiration
        a = (num & 0xff000000) >> 24
        b = (num & 0x00ff0000) >> 16
        c = (num & 0x0000ff00) >> 8
        d = num & 0x000000ff

        expirationBytes = [d, c, b, a]

        sb = ScriptBuilder()
        sb.EmitVarString(self.nexusName)
        sb.EmitVarString(self.chainName)
        sb.EmitVarInt(int(len(self.script) / 2))
        sb.AppendHexEncoded(self.script)
        sb.EmitBytes(expirationBytes)
        sb.EmitVarString(self.payload)

        if (withSignature):
            sb.EmitVarInt(len(self.signatures))
            for s in self.signatures:
                sb.AppendByte(1)
                sig = s.decode("utf-8").upper()
                sb.EmitVarInt(int(len(sig) / 2))
                sb.AppendHexEncoded(sig)

        return sb.data

    def getSign(self, msgHex: str, pk: int) -> bytes:
        '''Method that given a message to sign & HEX private key returns the signature needed.

        Args:
           msgHex (str): Message to sign as Hex String.
           pk (int): HEX private key. Example: 0x411d7dabb39b455aadc49897e2fa13234585116d4f5eee198105f33f8f62d0b8

        Returns:
            getSign (bytes): Signature as bytes.
        '''
        hexBytes = binascii.unhexlify(msgHex)
        hexPk = int(pk)

        cv = Curve.get_curve('Ed25519')
        pv_key = ECPrivateKey(hexPk, cv)
        signer = EDDSA(hashlib.sha512)

        sig = signer.sign(hexBytes, pv_key)
        sig = binascii.hexlify(sig)

        return(sig)
    
    def to_byte_array(self, with_signature):
        writer = PBinaryWriter()
        writer.write_string(self.nexusName)
        writer.write_string(self.chainName)
        writer.append_hex_encoded(self.script)
        writer.write_date_time(self.expiration)
        writer.append_hex_encoded(self.payload)
        if with_signature:
            writer.write_var_int(len(self.signatures))
            for sig in self.signatures:
                writer.write_signature(sig)

        return writer.to_uint8_array()
    

    def get_hash(self):
        # Convert the transaction to a string representation (without the signature)
        transaction_str = self.toString(False)

        # Compute SHA256 hash of the string
        hash_obj = hashlib.sha256(transaction_str.encode())

        # Convert the hash to a hexadecimal string and reverse it
        self.hash = hash_obj.hexdigest()[::-1]

        return self.hash
    
    def to_string(self, with_signature):
        parts = []

        # Append the nexus name and chain name
        parts.append(self.nexus_name)
        parts.append(self.chain_name)

        # Append the script (length and hex encoded)
        parts.append(str(len(self.script) // 2))
        parts.append(self.script)

        # Convert the expiration date to a timestamp and append
        expiration_timestamp = int(self.expiration.timestamp())
        parts.append(str(expiration_timestamp))

        # Append the payload (length and hex encoded)
        parts.append(str(len(self.payload) // 2))
        parts.append(self.payload)

        if with_signature:
            # Append the number of signatures
            parts.append(str(len(self.signatures)))
            for sig in self.signatures:
                # You need to adapt this part based on the structure of your Signature object in Python
                parts.append(str(sig.kind))
                parts.append(str(len(sig.bytes) // 2))
                parts.append(binascii.hexlify(sig.bytes).decode())

        return ''.join(parts)



# coding: utf-8

"""
    Phantasma API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ValidatorSettings(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'address': 'Address',
        'name': 'str',
        'host': 'str',
        'port': 'int',
        'url': 'str'
    }

    attribute_map = {
        'address': 'address',
        'name': 'name',
        'host': 'host',
        'port': 'port',
        'url': 'url'
    }

    def __init__(self, address=None, name=None, host=None, port=None, url=None):  # noqa: E501
        """ValidatorSettings - a model defined in Swagger"""  # noqa: E501
        self._address = None
        self._name = None
        self._host = None
        self._port = None
        self._url = None
        self.discriminator = None
        if address is not None:
            self.address = address
        if name is not None:
            self.name = name
        if host is not None:
            self.host = host
        if port is not None:
            self.port = port
        if url is not None:
            self.url = url

    @property
    def address(self):
        """Gets the address of this ValidatorSettings.  # noqa: E501


        :return: The address of this ValidatorSettings.  # noqa: E501
        :rtype: Address
        """
        return self._address

    @address.setter
    def address(self, address):
        """Sets the address of this ValidatorSettings.


        :param address: The address of this ValidatorSettings.  # noqa: E501
        :type: Address
        """

        self._address = address

    @property
    def name(self):
        """Gets the name of this ValidatorSettings.  # noqa: E501


        :return: The name of this ValidatorSettings.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ValidatorSettings.


        :param name: The name of this ValidatorSettings.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def host(self):
        """Gets the host of this ValidatorSettings.  # noqa: E501


        :return: The host of this ValidatorSettings.  # noqa: E501
        :rtype: str
        """
        return self._host

    @host.setter
    def host(self, host):
        """Sets the host of this ValidatorSettings.


        :param host: The host of this ValidatorSettings.  # noqa: E501
        :type: str
        """

        self._host = host

    @property
    def port(self):
        """Gets the port of this ValidatorSettings.  # noqa: E501


        :return: The port of this ValidatorSettings.  # noqa: E501
        :rtype: int
        """
        return self._port

    @port.setter
    def port(self, port):
        """Sets the port of this ValidatorSettings.


        :param port: The port of this ValidatorSettings.  # noqa: E501
        :type: int
        """

        self._port = port

    @property
    def url(self):
        """Gets the url of this ValidatorSettings.  # noqa: E501


        :return: The url of this ValidatorSettings.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this ValidatorSettings.


        :param url: The url of this ValidatorSettings.  # noqa: E501
        :type: str
        """

        self._url = url

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ValidatorSettings, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ValidatorSettings):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

"""
    Phantasma API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import apis into sdk package
from .API.account_api import AccountApi
from .API.auction_api import AuctionApi
from .API.block_api import BlockApi
from .API.chain_api import ChainApi
from .API.connection_api import ConnectionApi
from .API.contract_api import ContractApi
from .API.leaderboard_api import LeaderboardApi
from .API.nexus_api import NexusApi
from .API.organization_api import OrganizationApi
from .API.platform_api import PlatformApi
from .API.rpc_api import RpcApi
from .API.sale_api import SaleApi
from .API.storage_api import StorageApi
from .API.token_api import TokenApi
from .API.transaction_api import TransactionApi
from .API.validator_api import ValidatorApi
# import ApiClient
from .api_client import ApiClient
from .configuration import Configuration
# import models into sdk package
from .models.abi_event_result import ABIEventResult
from .models.abi_method_result import ABIMethodResult
from .models.abi_parameter_result import ABIParameterResult
from .models.account_result import AccountResult
from .models.address import Address
from .models.address_kind import AddressKind
from .models.archive_result import ArchiveResult
from .models.auction_result import AuctionResult
from .models.balance_result import BalanceResult
from .models.block_result import BlockResult
from .models.chain_result import ChainResult
from .models.contract_result import ContractResult
from .models.crowdsale_result import CrowdsaleResult
from .models.event_result import EventResult
from .models.governance_result import GovernanceResult
from .models.interop_result import InteropResult
from .models.leaderboard_result import LeaderboardResult
from .models.leaderboard_row_result import LeaderboardRowResult
from .models.net_info_peer import NetInfoPeer
from .models.net_info_peer_connection_status import NetInfoPeerConnectionStatus
from .models.net_info_peer_connection_status_channel import NetInfoPeerConnectionStatusChannel
from .models.net_info_peer_connection_status_monitor import NetInfoPeerConnectionStatusMonitor
from .models.nexus_result import NexusResult
from .models.node_info import NodeInfo
from .models.node_info_other import NodeInfoOther
from .models.node_info_protocol_version import NodeInfoProtocolVersion
from .models.oracle_result import OracleResult
from .models.organization_result import OrganizationResult
from .models.paginated_result import PaginatedResult
from .models.platform_result import PlatformResult
from .models.pub_key import PubKey
from .models.response_query import ResponseQuery
from .models.result_abci_query import ResultAbciQuery
from .models.result_health import ResultHealth
from .models.result_net_info import ResultNetInfo
from .models.result_status import ResultStatus
from .models.result_status_sync_info import ResultStatusSyncInfo
from .models.result_status_validator_info import ResultStatusValidatorInfo
from .models.rpc_request import RpcRequest
from .models.rpc_response import RpcResponse
from .models.script_result import ScriptResult
from .models.signature_result import SignatureResult
from .models.stake_result import StakeResult
from .models.storage_result import StorageResult
from .models.token_data_result import TokenDataResult
from .models.token_external_result import TokenExternalResult
from .models.token_price_result import TokenPriceResult
from .models.token_property_result import TokenPropertyResult
from .models.token_result import TokenResult
from .models.token_series_result import TokenSeriesResult
from .models.transaction_result import TransactionResult
from .models.validator_result import ValidatorResult
from .models.validator_settings import ValidatorSettings
